define("ckeditor/plugins/epalmostfullscreen/plugin", [
	"jquery",
	"ckeditor/ckeditor",
	"$dict!ep/dict"
], function ($, CKEDITOR, epDict) {

	/*
	 * @dictionary		ep.dict
	 *
	 * @translation		{Close}
	 */

	var container,

		overlay,

		area,

		toolbarContainer,

		openerInstance,

		editorInstance,

		offset = 30,

		contentTemp,

		statZIndex,

		translations = {
			tReduce: epDict.translate('Reduce'),
			tExpand: epDict.translate('Expand')
		},

		createContainer = function(){
			$('body').css({
				'overflow': 'hidden'
			});
			var winWidth = $(window).width(),
				winHeight = $(window).height(),
				docWidth = $(document).width(),
				docHeight = $(document).height(),
				toolbarMode = (openerInstance.config.pluginOptions && openerInstance.config.pluginOptions && openerInstance.config.pluginOptions.epalmostfullscreen && openerInstance.config.pluginOptions.epalmostfullscreen.toolbar) || 'Fullscreen';

			if(overlay){
				overlay.remove();
			}
			if(container){
				container.detach();
			}

			overlay = $('<div>')
				.appendTo('body')
				.css({
				'zIndex'   : statZIndex,
				'width' : docWidth + 'px',
				'height' : docHeight + 'px',
				'overflow': 'hidden'
				})
				.on('click',function(){
					closeEditor();
				})
				.addClass('ui-widget-overlay');

			container = $('<div>')
				.addClass('ep-cke-full ui-front')
				.appendTo('body')
				.css({
					'position' : 'absolute',
					'zIndex'   : statZIndex,
					'top' : (offset + $(document).scrollTop()) + 'px',
					'left' : offset + 'px',
					'width' : (winWidth - (2 * offset)) + 'px',
					'height' : (winHeight - (2 * offset)) + 'px'
				});

			if(area){
				area.remove();
			}

			toolbarContainer = $('<div>').attr('id','ep-cke-full-toolbar').appendTo(container);
			area = $('<textarea>').appendTo(container).css({
				'width' : '100%',
				'height' : '100%'
			});
			// set toolbar for BlogPostMode
			if(openerInstance.config.blogPostMode){
				toolbarMode = 'FullscreenPageBreak';
			};

			area.ckeditor(
				function(textarea){
					editorInstance = area.ckeditorGet();

					// Set name of original opener instance to handle anchors (language dependant) correctly
					editorInstance.originalName = openerInstance.name;

					editorInstance.setData(contentTemp);
					container.find('.cke_toolbox').children('span.cke_toolbar:last').css({
						'float': 'right'
					});

					//correct size of editor after creation to calculate correct size
					resizeEditor(editorInstance);
				},
				$.extend({
					sharedSpaces:{
						top : 'ep-cke-full-toolbar'
					},
					'height' : '100%', //create editor with 100% height will be changed after editor is created
					'toolbar' : toolbarMode,
					'baseFloatZIndex' : statZIndex + 1
				}, {
					pluginOptions : openerInstance.config.pluginOptions,
					useAbsolute: openerInstance.config.useAbsolute || false
				})
			);
		},

		closeEditor = function(){
			if(container && container.is(':visible')){
				$('body').css({
					'overflow': ''
				});
				container.hide();
				overlay.hide();

				if(openerInstance){
					// update textarea of opener editor when data are ready
					openerInstance.on('dataReady', function(evt){
						evt.editor.updateElement();
					});

					openerInstance.setData(editorInstance.getData());

					// destroy almost fullscreen instance once data is written back (avoid conflict on reopening)
					window.setTimeout(function () {
						editorInstance.destroy();
					}, 500);

				}
			}
		},

		resizeEditor = function(editor){
			var toolbarHeight = $('#ep-cke-full-toolbar').outerHeight(true);
			//width, height
			editor.resize( '100%', ($(window).height() - 2 * offset) - toolbarHeight );
		};

	$(window)
		.on('resize',function(){
			if(container && container.is(':visible')){
				var winWidth = $(window).width(),
					winHeight = $(window).height(),
					docWidth = $(document).width(),
					docHeight = $(document).height();
				overlay.css({
				'width' : docWidth + 'px',
				'height' : docHeight + 'px'
			})
				container.css({
					'top' : (offset + $(document).scrollTop()) + 'px',
					'left' : offset + 'px',
					'width' : (winWidth - (2 * offset)) + 'px',
					'height' : (winHeight - (2 * offset)) + 'px'
			});
				resizeEditor(editorInstance);
			}
		});

	CKEDITOR.plugins.add( 'epalmostfullscreen', {
		init: function( editor ){
			var lang = editor.lang;

			// define zIndex only once
			if(statZIndex === undefined){
				statZIndex = editor.config.baseFloatZIndex;
			}

			//command to open almost fullscreen mode
			editor.addCommand( 'epalmostfullscreen',{
					exec : function( editor ){
						contentTemp = editor.getData();
						openerInstance = editor;
						createContainer();
					}
				});
			//command to close editor
			editor.addCommand( 'epalmostfullscreenclose',{
					exec : function( editor ){
						closeEditor();
					}
				});
			//button to open fullscreen mode
			editor.ui.addButton( 'epalmostfullscreen',{
				label: translations.tExpand, //lang.maximize,
				command: 'epalmostfullscreen',
				icon: this.path + 'images/fullscreen.png'
			});
			//button to close fulllsreen mode and show that fullscreen is active
			editor.ui.addButton( 'epalmostfullscreenopen',{
				label: translations.tReduce,
				command: 'epalmostfullscreenclose',
				icon: this.path + 'images/fullscreen.png',
				className: 'cke_on'
			});
			//button to simulate close "x" on fullscreen mode
			editor.ui.addButton( 'epalmostfullscreenclose',{
				label: translations.tReduce,
				command: 'epalmostfullscreenclose',
				icon: this.path + 'images/close.png'
			});

		}
	});

	return CKEDITOR;

});
